/*
 * SSD1306/SSD1315に温湿度計を表示
 * 
 */

#include "AM2320Display.h"

static uint8_t graphBitPattern[] = {
  0x80,0xC0,0xE0,0xF0,0xF8,0xFC,0xFE,0xFF
} ;

// ----- 初期化 -----
void AM2320Display::init() {
  disp7seg.init();
  initGraphData() ;
}

// ---------------------
// ----- グラフ表示 -----
// ---------------------

// ----- グラフData初期化 ----
void AM2320Display::initGraphData() {
  thDataNum = 0 ;
  memset(thData,0,sizeof(thData) ) ;
}

// ----- グラフDataSet ----
void AM2320Display::addGraphData(int humidity,int temperature) {
  if (thDataNum == 89) {
    thData[thDataNum].temperature = temperature ; 
    thData[thDataNum].humidity = humidity ; 
    memcpy(thData,&thData[1],sizeof(struct thData_s) * 89) ;
  } else {
    thData[thDataNum].temperature = temperature ; 
    thData[thDataNum].humidity = humidity ; 
    thDataNum ++ ;
  }
}

// ----- グラフData表示 ----
void AM2320Display::dispGraphData() {
  uint8_t graphBuff[4][100] ;
  // ----- 
  disp7seg.disp7SEG(  0 , 0 , 11 ) ;
  disp7seg.disp7SEG(  0 , 4 , 13 ) ;
  
  int i,d,p,b,n,nd ;

  n = (thDataNum > 0 ) ? thDataNum -1 : 0 ; 
  int tip = thData[n].temperature ;
  int tipMin = tip ; 
  int tipMax = tip ;

  int hum = thData[n].humidity ;
  int humMin = hum ; 
  int humMax = hum ;

//  char str[120] ;
//  sprintf(str,"1:tip=%d tipMin=%d tipMax=%d hum=%d humMin=%d humMax=%d",tip,tipMin,tipMax,hum,humMin,humMax) ;
//  Serial.println(str) ;

  for (i = 0 ; i < thDataNum ; i++) {
    if (tipMin > thData[i].temperature) { tipMin = thData[i].temperature ; }
    if (tipMax < thData[i].temperature) { tipMax = thData[i].temperature ; }

    if (humMin > thData[i].humidity ) { humMin = thData[i].humidity ; }
    if (humMax < thData[i].humidity ) { humMax = thData[i].humidity ; }
  }
//  sprintf(str,"2:tip=%d tipMin=%d tipMax=%d hum=%d humMin=%d humMax=%d",tip,tipMin,tipMax,hum,humMin,humMax) ;
//  Serial.println(str) ;

  // ----- 気温 -----
  memset(graphBuff,0,sizeof(graphBuff)) ;

  if (tipMax - tipMin <= 10) {
    tipMax = (tipMax+tipMin) / 2  + 5; 
    tipMin = tipMax - 10 ;
  }
//  sprintf(str,"3:tip=%d tipMin=%d tipMax=%d",tip,tipMin,tipMax) ;
//  Serial.println(str) ;

  disp7seg.disp7GraphFont(  17 , 0 , ('0' + tipMax / 10) ) ;
  disp7seg.disp7GraphFont(  22 , 0 , ('0' + tipMax % 10) ) ;

  disp7seg.disp7GraphFont(  17 , 3 , ('0' + tipMin / 10) ) ;
  disp7seg.disp7GraphFont(  22 , 3 , ('0' + tipMin % 10) ) ;

  graphBuff[0][4] = 0xFF ;
  graphBuff[1][4] = 0xFF ;
  graphBuff[2][4] = 0xFF ;
  graphBuff[3][4] = 0xFF ;

  // --- tipMin
  for (i = 0 ; i < 100 ; i++) {
    graphBuff[0][i] ^= 0x08 ;
  }

  // --- tipMax
  for (i = 0 ; i < 100 ; i++) {
    graphBuff[3][i] ^= 0x10 ;
  }

  int tipRange = tipMax - tipMin ;
  for (i = 0 ; i < thDataNum ; i++) {
    n = thData[i].temperature ;
    if (tipMin <= n && n <= tipMax) {
      d = ((n-tipMin)*10.) / tipRange * 2.4 + 4 ;
      p = 3 - ((d / 8) % 4) ;
      b = d % 8 ;
      // ---- BitPattern設定
      uint8_t bitData = graphBitPattern[b] ;
      if (p == 3) { bitData &= 0x0F ; }
      if (p <= 2) { graphBuff[3][i+5] = 0x0F ; }
      if (p <= 1) { graphBuff[2][i+5] = 0xFF ; }
      if (p == 0) { graphBuff[1][i+5] = 0xFF ; bitData &= 0xF0 ;}
      graphBuff[p][i+5] ^= bitData ;
//      graphBuff[p][i+5] ^= 0x80 >> b ;
    }
  }
//  sprintf(str,"T:thDataNum=%d N=%d d=%d p=%d b=%d",thDataNum,n,d,p,b) ;
//  Serial.println(str) ;
  disp7seg.oledBitMap(28,0,100,4,(uint8_t *)&graphBuff[0][0]) ;


  // -- 湿度
  memset(graphBuff,0,sizeof(graphBuff)) ;
  if (humMax - humMin <= 10) {
    humMax = (humMax+humMin) / 2  + 5; 
    humMin = humMax - 10 ;
  }
//  sprintf(str,"3:hum=%d humMin=%d humMax=%d",hum,humMin,humMax) ;
//  Serial.println(str) ;

  disp7seg.disp7GraphFont(  17 , 4 , ('0' + humMax / 10) ) ;
  disp7seg.disp7GraphFont(  22 , 4 , ('0' + humMax % 10) ) ;

  disp7seg.disp7GraphFont(  17 , 7 , ('0' + humMin / 10) ) ;
  disp7seg.disp7GraphFont(  22 , 7 , ('0' + humMin % 10) ) ;

  graphBuff[0][4] = 0xFF ;
  graphBuff[1][4] = 0xFF ;
  graphBuff[2][4] = 0xFF ;
  graphBuff[3][4] = 0xFF ;

  // --- tipMin
  for (i = 0 ; i < 100 ; i++) {
    graphBuff[0][i] ^= 0x08 ;
  }

  // --- tipMax
  for (i = 0 ; i < 100 ; i++) {
    graphBuff[3][i] ^= 0x10 ;
  }

  int humRange = humMax - humMin ;
  for (i = 0 ; i < thDataNum ; i++) {
    n = thData[i].humidity ;
    if (humMin <= n && n <= humMax) {
      d = ((n-humMin)*10.) / humRange * 2.4 + 4 ;
      p = 3 - ((d / 8) % 4) ;
      b = d % 8 ;
      // ---- BitPattern設定
      uint8_t bitData = graphBitPattern[b] ;
      if (p == 3) { bitData &= 0x0F ; }
      if (p <= 2) { graphBuff[3][i+5] = 0x0F ; }
      if (p <= 1) { graphBuff[2][i+5] = 0xFF ; }
      if (p == 0) { graphBuff[1][i+5] = 0xFF ;  bitData &= 0xF0 ; }
      graphBuff[p][i+5] ^= bitData ;
//      graphBuff[p][i+5] ^= 0x80 >> b ;
    }
  }
//  sprintf(str,"H:thDataNum=%d N=%d d=%d p=%d b=%d",thDataNum,n,d,p,b) ;
//  Serial.println(str) ;
  disp7seg.oledBitMap(28,4,100,4,(uint8_t *)&graphBuff[0][0]) ;
  
  
  disp7seg.dispSwitch(true) ;
}

// ---------------------------------
// ----- 7セグメントLEDッポイ表示 -----
// ---------------------------------
/**
 * uint8_t humidityN 湿度 - 整数部
 * uint8_t temperatureN 温度 - 整数部
 * uint8_t temperatureD 温度 - 小数部
 */
void AM2320Display::disp7segLED(int humidity,int temperature) {
      // --- 7セグメントLEDっぽい表示 ---
      int t0 = abs(temperature) / 10 ;
      int t3 = abs(temperature) % 10 ;
      int t1 = t0 / 10 ;
      int t2 = t0 % 10 ;
      if (temperature < 0) { t1 = -1; }
  
      int h0 = humidity / 10 ;
      int h3 = humidity % 10 ;
      int h1 = h0 / 10 ;
      int h2 = h0 % 10 ;
  
      // ----- SSD1306に表示 -----
      disp7seg.disp7SEG(  0 , 0 , 11 ) ;
      disp7seg.disp7SEG( 20 , 0 , t1 ) ;
      disp7seg.disp7SEG( 40 , 0 , t2 ) ;
      disp7seg.disp7SEG( 56 , 0 , 10 ) ;
      disp7seg.disp7SEG( 72 , 0 , t3 ) ;
      disp7seg.disp7SEG( 92 , 0 , 12 ) ;
    
      disp7seg.disp7SEG(  0 , 4 , 13 ) ;
      disp7seg.disp7SEG( 20 , 4 , h1 ) ;
      disp7seg.disp7SEG( 40 , 4 , h2 ) ;
      disp7seg.disp7SEG( 56 , 4 , 10 ) ;
      disp7seg.disp7SEG( 72 , 4 , h3 ) ;
      disp7seg.disp7SEG( 92 , 4 , 14 ) ;
      disp7seg.dispSwitch(true) ;
}
